import seaborn as sns
import matplotlib.pyplot as plt
from sklearn.metrics import confusion_matrix
from .TextProcess import text_tokens
from .Scores import single_label_f_score
from tqdm import tqdm

tqdm.pandas()
import nltk as nltk
from sklearn.model_selection import train_test_split
from nltk.classify import apply_features


def process_sow(text):
    Text = text_tokens(
        text,
        lower_bound_percentage=0,
        higher_bound_percentage=1,
        minimal_word_length=0,
        remove_punctuations=False,
        remove_non_letter_characters=True,
        lemmatize_the_words=False,
        stemmer_the_words=True,
        part_of_speech_filter=False,
        english_text_filter=False,
        stop_words_filter=True,
        other_words_filter=False,
        remove_adjacent_tokens=False,
        tokens_form=False
    )
    return Text


def process_sow_quick(text):
    Text = text_tokens(
        text,
        lower_bound_percentage=0,
        higher_bound_percentage=1,
        minimal_word_length=0,
        remove_punctuations=False,
        remove_non_letter_characters=False,
        lemmatize_the_words=False,
        stemmer_the_words=False,
        part_of_speech_filter=False,
        english_text_filter=False,
        stop_words_filter=False,
        other_words_filter=False,
        remove_adjacent_tokens=False,
        tokens_form=False
    )
    return Text


def Naive_Bayes_classify(
    input_df,
    feature_column,
    label_column,
    process_text=False,
    test_size=.2,
    random_state=1,
    feature_fct=None,
    most_informative_features=10,
    show_info=True,
    return_fscore=True
):
    df = input_df.copy()
    if process_text:
        df[feature_column] = df[feature_column].progress_apply(process_sow)
    else:
        df[feature_column] = df[feature_column].apply(process_sow_quick)
    features = list(df[feature_column])
    labels = list(df[label_column])
    data_set = [(f, labels[index]) for index, f in enumerate(features)]
    if feature_fct:
        get_features = feature_fct
    else:

        def get_features(text):
            features = {}
            tokens = text.split(' ')
            features["sow length"] = len(tokens)
            fd = nltk.FreqDist(tokens)
            most_common = fd.most_common(3)
            top = most_common[0][0]
            sec = most_common[1][0]
            thr = most_common[2][0]
            features["1st word"] = top
            features["2nd word"] = sec
            features["3rd word"] = thr
            return features

    X_train, X_test = train_test_split(data_set, test_size=test_size, random_state=random_state)
    train_set = apply_features(get_features, X_train)
    test_set = apply_features(get_features, X_test)
    classifier = nltk.NaiveBayesClassifier.train(train_set)
    y_pred = []
    y_test = []
    for i in range(len(list(test_set))):
        y_pred.append(classifier.classify(test_set[i][0]))
        y_test.append(test_set[i][1])
    f, cf = single_label_f_score(y_gold=y_test, y_pred=y_pred)
    if show_info:
        print('f-score:', f)
        print('label wise f-score', cf)
        print(classifier.show_most_informative_features(most_informative_features))
        conf_mat = confusion_matrix(y_test, y_pred)
        fig, ax = plt.subplots(figsize=(4, 4))
        labels = list(set(labels))
        sns.heatmap(
            conf_mat, annot=True, cmap="Blues", fmt='d', xticklabels=labels, yticklabels=labels
        )
        plt.ylabel('Actual')
        plt.xlabel('Predicted')
        plt.title("NaiveBayes CONFUSION MATRIX", size=16)
    if return_fscore:
        return f, cf



from tqdm import tqdm

tqdm.pandas()
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from sklearn.metrics import confusion_matrix
from .TextProcess import text_tokens
from .Scores import single_label_f_score
from sklearn.model_selection import train_test_split
import fasttext


def process_sow(text):
    Text = text_tokens(
        text,
        lower_bound_percentage=0,
        higher_bound_percentage=1,
        minimal_word_length=0,
        remove_punctuations=False,
        remove_non_letter_characters=True,
        lemmatize_the_words=False,
        stemmer_the_words=True,
        part_of_speech_filter=False,
        english_text_filter=False,
        stop_words_filter=True,
        other_words_filter=False,
        remove_adjacent_tokens=False,
        tokens_form=False
    )
    return Text


def process_sow_quick(text):
    Text = text_tokens(
        text,
        lower_bound_percentage=0,
        higher_bound_percentage=1,
        minimal_word_length=0,
        remove_punctuations=False,
        remove_non_letter_characters=False,
        lemmatize_the_words=False,
        stemmer_the_words=False,
        part_of_speech_filter=False,
        english_text_filter=False,
        stop_words_filter=False,
        other_words_filter=False,
        remove_adjacent_tokens=False,
        tokens_form=False
    )
    return Text


def to_txt(data, label, name, mode='w'):
    Data = []
    for index, d in enumerate(data):
        Data.append(['__label__' + str(label[index]), d])
    df = pd.DataFrame(Data, columns=['Token', 'Statement'])
    df.to_csv(r'{}'.format(name), header=None, index=None, sep=' ', mode=mode)


def fasttext_classify(
    input_df,
    feature_column,
    label_column,
    process_text=False,
    test_size=.2,
    random_state=1,
    word_dim=100,
    epoch=10,
    show_info=True,
    return_fscore=True
):
    df = input_df.copy()
    if process_text:
        df[feature_column] = df[feature_column].progress_apply(process_sow)
    else:
        df[feature_column] = df[feature_column].apply(process_sow_quick)
    features = list(df[feature_column])
    labels = list(df[label_column])
    labels = [str(l) for l in labels]
    X_train, X_test, y_train, y_test = train_test_split(
        features, labels, test_size=test_size, random_state=random_state
    )
    to_txt(X_train, y_train, f'fasttext_train.txt', mode='w')
    model = fasttext.train_supervised(
        f'fasttext_train.txt',
        minCount=1,
        minCountLabel=1,
        wordNgrams=1,
        lr=.5,
        lrUpdateRate=100,
        dim=word_dim,
        ws=5,
        minn=3,
        maxn=6,
        epoch=epoch
    )
    y_pred = [model.predict(text)[0][0][9:] for text in X_test]
    f, cf = single_label_f_score(y_gold=y_test, y_pred=y_pred)
    if show_info:
        print('f-score:', f)
        print('label wise f-score', cf)
        conf_mat = confusion_matrix(y_test, y_pred)
        fig, ax = plt.subplots(figsize=(4, 4))
        labels = list(set(labels))
        sns.heatmap(
            conf_mat, annot=True, cmap="Blues", fmt='d', xticklabels=labels, yticklabels=labels
        )
        plt.ylabel('Actual')
        plt.xlabel('Predicted')
        plt.title("FASTTEXT CONFUSION MATRIX", size=16)
    if return_fscore:
        return f, cf



from pickle import load, dump
import numpy as np


def read_pickle(filename):
    # the filename should mention the extension 'pk1'
    input = open(filename, 'rb')
    Whatever = load(input)
    input.close()
    return Whatever


def save_pickle(stuff, savename):
    output = open(savename, 'wb')
    dump(stuff, output, -1)
    output.close()


def saveList(myList, savename):
    np.save(savename, myList)


def loadList(filename, path):
    # the filename should mention the extension 'npy'
    tempNumpyArray = np.load(path + filename)
    return tempNumpyArray.tolist()



import boto3
import logging
from tqdm import tqdm
import os
from botocore.exceptions import ClientError
from boto.s3.connection import S3Connection


def splitall(path):
    allparts = []
    while 1:
        parts = os.path.split(path)
        if parts[0] == path:  # sentinel for absolute paths
            allparts.insert(0, parts[0])
            break
        elif parts[1] == path:  # sentinel for relative paths
            allparts.insert(0, parts[1])
            break
        else:
            path = parts[0]
            allparts.insert(0, parts[1])
    return allparts


def hook(t):
    def inner(bytes_amount):
        t.update(bytes_amount)

    return inner


def S3_download(file_name, save_file_name, bucket='bonfire-data-team-share', show_progress=True):
    if show_progress:
        s3 = boto3.resource('s3')
        s31 = boto3.client('s3')
        file_object = s3.Object(bucket, file_name)
        filesize = file_object.content_length
        with tqdm(total=filesize, unit='B', unit_scale=True, desc=file_name) as t:
            s31.download_file(bucket, file_name, save_file_name, Callback=hook(t))
    else:
        s3 = boto3.client('s3')
        s3.download_file(bucket, file_name, save_file_name)


def upload_file(
    file_name, save_file_name=None, bucket='bonfire-data-team-share', show_progress=True
):
    """Upload a file to an S3 bucket

    :param file_name: File to upload
    :param bucket: Bucket to upload to
    :param object_name: S3 object name. If not specified then file_name is used
    :return: True if file was uploaded, else False
    """
    # If S3 object_name was not specified, use file_name
    if save_file_name is None:
        save_file_name = file_name
    # Upload the file
    s3_client = boto3.client('s3')
    try:
        if show_progress:
            with tqdm(total=float(os.path.getsize(file_name)), unit='B', unit_scale=True,
                      desc=file_name) as t:
                s3_client.upload_file(file_name, bucket, save_file_name, Callback=hook(t))
        else:
            s3_client.upload_file(file_name, bucket, save_file_name)
    except ClientError as e:
        logging.error(e)
        return False
    return True


def S3_download_folder(credentials_dict, folder_name, bucket_name, save_path=None):
    """
    This function is able to download a single file or a whole folder, but it requires the credentials

    @param credentials_dict: {'Access key ID': your Access key ID, 'Secret access key': your secret access key}
    @param folder_name: your remote folder name or file name, need to be this format: folder/sub
    @param bucket_name: bucket_name is a string or your bucket name
    @param save_path: save path is where you want to save the folder or file, default is your current working directory

    """
    if save_path:
        pass
    else:
        save_path = os.getcwd()
    conn = S3Connection(credentials_dict['Access key ID'], credentials_dict['Secret access key'])
    bucket = conn.get_bucket(bucket_name)
    folder_content = []
    for key in bucket.list():
        diretory_name = key.name
        if diretory_name.startswith(folder_name):
            # name_parse=diretory_name.split('/')
            name_parse = splitall(diretory_name)
            if not name_parse[-1]:
                try:
                    os.mkdir(os.path.join(save_path, diretory_name))
                except:
                    pass
            else:
                for i in range(1, len(name_parse)):
                    try:
                        os.mkdir(os.path.join(save_path, *name_parse[:i]))
                    except:
                        pass
                folder_content.append(diretory_name)
    s3 = boto3.resource('s3')
    s31 = boto3.client('s3')
    for file_name in folder_content:
        file_object = s3.Object(bucket_name, file_name)
        filesize = file_object.content_length
        with tqdm(total=filesize, unit='B', unit_scale=True, desc=file_name) as t:
            s31.download_file(
                bucket_name, file_name, os.path.join(save_path, file_name), Callback=hook(t)
            )


def S3_upload_folder(
    foler_name, remote_folder, bucket, ignore_files=['.DS_Store'], show_progress=True
):
    """
    This function is able to upload a folder or a file to specific place on S3
    @param foler_name: need to be like this format: /Users/xinzhang/folder/sub
                       or a file like : /Users/xinzhang/folder/sub/data.csv
    @param remote_folder: like this on S3 root/folder/sub
    @param bucket: your bucket, string
    @param ignore_files: a list of string
    @param show_progress: Boolean
    """
    if list(os.walk(foler_name)):
        actuall_folder_name = splitall(foler_name)[-1]
        pre_path = os.path.join(*splitall(foler_name)[:-1])
        for path, subdirs, files in os.walk(foler_name):
            for name in files:
                if name in ignore_files:
                    continue
                full_path = os.path.join(path, name)
                folders = []
                while os.path.split(full_path)[1] != actuall_folder_name:
                    folders.append(os.path.split(full_path)[1])
                    full_path = os.path.split(full_path)[0]
                folders.append(actuall_folder_name)
                folders.reverse()
                file_path = os.path.join(*folders)
                aws_path = os.path.join(remote_folder, file_path)
                upload_file(
                    os.path.join(pre_path, file_path),
                    save_file_name=aws_path,
                    bucket=bucket,
                    show_progress=show_progress
                )
    else:
        file_name = splitall(foler_name)[-1]
        aws_path = os.path.join(remote_folder, file_name)
        upload_file(foler_name, save_file_name=aws_path, bucket=bucket, show_progress=show_progress)
